@@ -47,6 +47,7 @@ from tensorflow_estimator.python.estimator import model_fn
 from tensorflow_estimator.python.estimator.canned import metric_keys
 from tensorflow_estimator.python.estimator.canned import prediction_keys
 from tensorflow_estimator.python.estimator.export import export_output
+from tensorflow_estimator.python.estimator.mode_keys import ModeKeys
 
 _DEFAULT_SERVING_KEY = signature_constants.DEFAULT_SERVING_SIGNATURE_DEF_KEY
 
@@ -145,7 +146,7 @@ class _Head(object):
         mode=mode,
         logits=logits,
         train_op_fn=lambda _: tf.no_op())
-    if mode == model_fn.ModeKeys.TRAIN:
+    if mode == ModeKeys.TRAIN:
       optimizer = ...
       sync = tf.train.SyncReplicasOptimizer(opt=optimizer, ...)
       update_op = sync.minimize(
@@ -441,7 +442,8 @@ def _check_logits_final_dim(logits, expected_logits_dimension):
     with ops.control_dependencies([assert_rank]):
       static_shape = logits.shape
       if static_shape.ndims is not None and static_shape[-1] is not None:
-        if static_shape[-1] != expected_logits_dimension:
+        if (isinstance(expected_logits_dimension, int)
+            and static_shape[-1] != expected_logits_dimension):
           raise ValueError(
               'logits shape must be [D0, D1, ... DN, logits_dimension], '
               'got %s.' % (static_shape,))
@@ -475,6 +477,31 @@ def _validate_loss_fn_args(loss_fn):
     raise ValueError('loss_fn has unexpected args: {}'.format(invalid_args))
 
 
+def _validate_n_classes(n_classes):
+  """Validates n_classes argument.
+
+  Required arguments: n_classes.
+
+  Args:
+    n_classes: The number of classes.
+  Raises:
+    ValueError: If n_classes is <= 2 and n_classes is a Python integer.
+  Returns:
+    n_classes in its original type.
+  """
+  if isinstance(n_classes, int) and (n_classes <= 2):
+      raise ValueError('n_classes must be > 2: %s.' % n_classes)
+
+  n_classes_as_tensor = ops.convert_to_tensor(n_classes)
+  assert_n_classes = check_ops.assert_greater(
+      n_classes_as_tensor, 2, message='n_classes must be greater than 2')
+  with ops.control_dependencies([assert_n_classes]):
+    control_flow_ops.no_op()
+  # Return n_classes in its original type, so that any code
+  # using the accessor logits_dimension() has the original type.
+  return n_classes
+
+
 def _call_loss_fn(loss_fn, labels, logits, features, expected_loss_dim=1):
   """Calls loss_fn and checks the returned shape.
 
@@ -519,6 +546,25 @@ def _indicator_labels_mean(labels, weights=None, name=None):
     return metrics_lib.mean(labels, weights=weights, name=scope)
 
 
+def _all_class_ids(logits, n_classes):
+  batch_size = array_ops.shape(logits)[0]
+  class_id_list = math_ops.range(n_classes)
+  return array_ops.tile(
+      input=array_ops.expand_dims(input=class_id_list, axis=0),
+      multiples=[batch_size, 1])
+
+
+def _all_classes(logits, n_classes, label_vocabulary=None):
+  batch_size = array_ops.shape(logits)[0]
+  if label_vocabulary:
+    classes_list = label_vocabulary
+  else:
+    classes_list = string_ops.as_string(math_ops.range(n_classes))
+  return array_ops.tile(
+      input=array_ops.expand_dims(input=classes_list, axis=0),
+      multiples=[batch_size, 1])
+
+
 def _classification_output(scores, n_classes, label_vocabulary=None):
   batch_size = array_ops.shape(scores)[0]
   if label_vocabulary:
@@ -686,9 +732,9 @@ class _MultiClassHeadWithSoftmaxCrossEntropyLoss(_Head):
                loss_reduction=losses.Reduction.SUM,
                loss_fn=None,
                name=None):
-    if (n_classes is None) or (n_classes <= 2):
-      raise ValueError('n_classes must be > 2: %s.' % n_classes)
-    self._n_classes = n_classes
+    if n_classes is None:
+      raise ValueError('n_classes cannot be None')
+    self._n_classes = _validate_n_classes(n_classes)
     self._weight_column = weight_column
     self._label_vocabulary = label_vocabulary
     self._loss_reduction = loss_reduction
@@ -795,8 +841,7 @@ class _MultiClassHeadWithSoftmaxCrossEntropyLoss(_Head):
       regularization_losses: A list of additional scalar losses to be added to
         the training loss, such as regularization losses. These losses are
         usually expressed as a batch average, so for best results users need to
-        set `loss_reduction=SUM_OVER_BATCH_SIZE` or
-        `loss_reduction=SUM_OVER_NONZERO_WEIGHTS` when creating the head to
+        set `loss_reduction=SUM_OVER_BATCH_SIZE` when creating the head to
         avoid scaling errors.
     Returns:
       A `model_fn._TPUEstimatorSpec` instance.
@@ -810,6 +855,9 @@ class _MultiClassHeadWithSoftmaxCrossEntropyLoss(_Head):
       # Predict.
       pred_keys = prediction_keys.PredictionKeys
       with ops.name_scope(None, 'predictions', (logits,)):
+        all_class_ids = _all_class_ids(logits, self._n_classes)
+        all_classes = _all_classes(logits, self._n_classes,
+                                   label_vocabulary=self._label_vocabulary)
         # class_ids's shape is [D0, D1, ... DN].
         class_ids = math_ops.argmax(logits, axis=-1, name=pred_keys.CLASS_IDS)
         class_ids = array_ops.expand_dims(class_ids, axis=-1)
@@ -828,13 +876,15 @@ class _MultiClassHeadWithSoftmaxCrossEntropyLoss(_Head):
             # Expand to [batch_size, 1]
             pred_keys.CLASS_IDS: class_ids,
             pred_keys.CLASSES: classes,
+            pred_keys.ALL_CLASS_IDS: all_class_ids,
+            pred_keys.ALL_CLASSES: all_classes,
         }
-      if mode == model_fn.ModeKeys.PREDICT:
+      if mode == ModeKeys.PREDICT:
         classifier_output = _classification_output(
             scores=probabilities, n_classes=self._n_classes,
             label_vocabulary=self._label_vocabulary)
         return model_fn._TPUEstimatorSpec(  # pylint: disable=protected-access
-            mode=model_fn.ModeKeys.PREDICT,
+            mode=ModeKeys.PREDICT,
             predictions=predictions,
             export_outputs={
                 _DEFAULT_SERVING_KEY: classifier_output,
@@ -852,18 +902,19 @@ class _MultiClassHeadWithSoftmaxCrossEntropyLoss(_Head):
         regularization_loss = None
         regularized_training_loss = training_loss
       # Eval.
-      if mode == model_fn.ModeKeys.EVAL:
+      if mode == ModeKeys.EVAL:
         return model_fn._TPUEstimatorSpec(  # pylint: disable=protected-access
-            mode=model_fn.ModeKeys.EVAL,
+            mode=ModeKeys.EVAL,
             predictions=predictions,
             loss=regularized_training_loss,
-            eval_metrics=_create_eval_metrics_tuple(self._eval_metric_ops, {
-                'labels': label_ids,
-                'class_ids': class_ids,
-                'weights': weights,
-                'unreduced_loss': unreduced_loss,
-                'regularization_loss': regularization_loss
-            }))
+            eval_metrics=_create_eval_metrics_tuple(
+                self._eval_metric_ops, {
+                    'labels': label_ids,
+                    'class_ids': class_ids,
+                    'weights': weights,
+                    'unreduced_loss': unreduced_loss,
+                    'regularization_loss': regularization_loss
+                }))
 
       # Train.
       if optimizer is not None:
@@ -899,7 +950,7 @@ class _MultiClassHeadWithSoftmaxCrossEntropyLoss(_Head):
             _summary_key(self._name, keys.LOSS_REGULARIZATION),
             regularization_loss)
     return model_fn._TPUEstimatorSpec(  # pylint: disable=protected-access
-        mode=model_fn.ModeKeys.TRAIN,
+        mode=ModeKeys.TRAIN,
         predictions=predictions,
         loss=regularized_training_loss,
         train_op=train_op)
@@ -1001,7 +1052,7 @@ class _BinaryLogisticHeadWithSigmoidCrossEntropyLoss(_Head):
                loss_fn=None,
                name=None):
     self._weight_column = weight_column
-    self._thresholds = thresholds
+    self._thresholds = tuple(thresholds) if thresholds else tuple()
     self._label_vocabulary = label_vocabulary
     self._loss_reduction = loss_reduction
     self._loss_fn = loss_fn
@@ -1155,8 +1206,7 @@ class _BinaryLogisticHeadWithSigmoidCrossEntropyLoss(_Head):
       regularization_losses: A list of additional scalar losses to be added to
         the training loss, such as regularization losses. These losses are
         usually expressed as a batch average, so for best results users need to
-        set `loss_reduction=SUM_OVER_BATCH_SIZE` or
-        `loss_reduction=SUM_OVER_NONZERO_WEIGHTS` when creating the head to
+        set `loss_reduction=SUM_OVER_BATCH_SIZE` when creating the head to
         avoid scaling errors.
     Returns:
       `EstimatorSpec`.
@@ -1178,6 +1228,10 @@ class _BinaryLogisticHeadWithSigmoidCrossEntropyLoss(_Head):
         class_ids = math_ops.argmax(
             two_class_logits, axis=-1, name=pred_keys.CLASS_IDS)
         class_ids = array_ops.expand_dims(class_ids, axis=-1)
+        all_class_ids = _all_class_ids(logits, n_classes=2)
+        all_classes = _all_classes(logits, n_classes=2,
+                                   label_vocabulary=self._label_vocabulary)
+
         if self._label_vocabulary:
           table = lookup_ops.index_to_string_table_from_tensor(
               vocabulary_list=self._label_vocabulary,
@@ -1191,13 +1245,15 @@ class _BinaryLogisticHeadWithSigmoidCrossEntropyLoss(_Head):
             pred_keys.PROBABILITIES: probabilities,
             pred_keys.CLASS_IDS: class_ids,
             pred_keys.CLASSES: classes,
+            pred_keys.ALL_CLASS_IDS: all_class_ids,
+            pred_keys.ALL_CLASSES: all_classes,
         }
-      if mode == model_fn.ModeKeys.PREDICT:
+      if mode == ModeKeys.PREDICT:
         classifier_output = _classification_output(
             scores=probabilities, n_classes=2,
             label_vocabulary=self._label_vocabulary)
         return model_fn._TPUEstimatorSpec(  # pylint: disable=protected-access
-            mode=model_fn.ModeKeys.PREDICT,
+            mode=ModeKeys.PREDICT,
             predictions=predictions,
             export_outputs={
                 _DEFAULT_SERVING_KEY: classifier_output,
@@ -1219,14 +1275,13 @@ class _BinaryLogisticHeadWithSigmoidCrossEntropyLoss(_Head):
         regularized_training_loss = training_loss
 
       # Eval.
-      if mode == model_fn.ModeKeys.EVAL:
+      if mode == ModeKeys.EVAL:
         return model_fn._TPUEstimatorSpec(  # pylint: disable=protected-access
-            mode=model_fn.ModeKeys.EVAL,
+            mode=ModeKeys.EVAL,
             predictions=predictions,
             loss=regularized_training_loss,
             eval_metrics=_create_eval_metrics_tuple(
-                self._eval_metric_ops,
-                {
+                self._eval_metric_ops, {
                     'labels': processed_labels,
                     'logits': logits,
                     'logistic': logistic,
@@ -1234,8 +1289,7 @@ class _BinaryLogisticHeadWithSigmoidCrossEntropyLoss(_Head):
                     'weights': weights,
                     'unreduced_loss': unreduced_loss,
                     'regularization_loss': regularization_loss
-                }
-            ))
+                }))
 
       # Train.
       if optimizer is not None:
@@ -1270,7 +1324,7 @@ class _BinaryLogisticHeadWithSigmoidCrossEntropyLoss(_Head):
             _summary_key(self._name, keys.LOSS_REGULARIZATION),
             regularization_loss)
     return model_fn._TPUEstimatorSpec(  # pylint: disable=protected-access
-        mode=model_fn.ModeKeys.TRAIN,
+        mode=ModeKeys.TRAIN,
         predictions=predictions,
         loss=regularized_training_loss,
         train_op=train_op)
@@ -1447,8 +1501,7 @@ class _RegressionHeadWithMeanSquaredErrorLoss(_Head):
       regularization_losses: A list of additional scalar losses to be added to
         the training loss, such as regularization losses. These losses are
         usually expressed as a batch average, so for best results users need to
-        set `loss_reduction=SUM_OVER_BATCH_SIZE` or
-        `loss_reduction=SUM_OVER_NONZERO_WEIGHTS` when creating the head to
+        set `loss_reduction=SUM_OVER_BATCH_SIZE` when creating the head to
         avoid scaling errors.
     Returns:
       A `model_fn._TPUEstimatorSpec` instance.
@@ -1469,11 +1522,11 @@ class _RegressionHeadWithMeanSquaredErrorLoss(_Head):
         predicted_value = logits
         predictions = {
             prediction_keys.PredictionKeys.PREDICTIONS: predicted_value}
-      if mode == model_fn.ModeKeys.PREDICT:
+      if mode == ModeKeys.PREDICT:
         regression_output = export_output.RegressionOutput(
             value=predicted_value)
         return model_fn._TPUEstimatorSpec(  # pylint: disable=protected-access
-            mode=model_fn.ModeKeys.PREDICT,
+            mode=ModeKeys.PREDICT,
             predictions=predictions,
             export_outputs={
                 _DEFAULT_SERVING_KEY: regression_output,
@@ -1492,9 +1545,9 @@ class _RegressionHeadWithMeanSquaredErrorLoss(_Head):
         regularized_training_loss = training_loss
 
       # Eval.
-      if mode == model_fn.ModeKeys.EVAL:
+      if mode == ModeKeys.EVAL:
         return model_fn._TPUEstimatorSpec(  # pylint: disable=protected-access
-            mode=model_fn.ModeKeys.EVAL,
+            mode=ModeKeys.EVAL,
             predictions=predictions,
             loss=regularized_training_loss,
             eval_metrics=_create_eval_metrics_tuple(
@@ -1539,7 +1592,7 @@ class _RegressionHeadWithMeanSquaredErrorLoss(_Head):
             _summary_key(self._name, keys.LOSS_REGULARIZATION),
             regularization_loss)
     return model_fn._TPUEstimatorSpec(  # pylint: disable=protected-access
-        mode=model_fn.ModeKeys.TRAIN,
+        mode=ModeKeys.TRAIN,
         predictions=predictions,
         loss=regularized_training_loss,
         train_op=train_op)
