@@ -21,6 +21,12 @@ from __future__ import print_function
 from contextlib import contextmanager
 import copy
 
+from tensorflow.python.distribute import distribute_lib
+from tensorflow.python.distribute import distribution_strategy_context
+from tensorflow.python.eager import context as eager_context
+from tensorflow.python.framework import constant_op
+from tensorflow.python.ops import summary_ops_v2
+from tensorflow.python.ops import math_ops
 from tensorflow.python.platform import tf_logging as logging
 from tensorflow.python.tpu import device_assignment as tpu_device_assignment
 from tensorflow.python.tpu import tpu_system_metadata as tpu_system_metadata_lib
@@ -38,6 +44,9 @@ _NUM_CORES_TO_COMPUTATION_SHAPE = {
     4: [1, 2, 2],
     8: [2, 2, 2],
     16: [4, 2, 2],
+    32: [4, 4, 2],
+    64: [8, 4, 2],
+    128: [8, 8, 2],
 }
 
 
@@ -79,7 +88,7 @@ class TPUContext(object):
            current invocation is 1. For per-host v2 input pipeline deployment,
            total invocation count is equal to the number of hosts in the system
            and num replicas consumed by current invocation is equal to number of
-           cores per host.
+           replicas per host.
 
     Raises:
       RuntimeError: If this method must not be called from input_fn.
@@ -95,6 +104,9 @@ class TPUContext(object):
     elif self._internal_ctx.is_input_broadcast_with_iterators():
       total_invocation_count = 1
       replicas_consumed = self._internal_ctx.num_replicas
+    elif self._internal_ctx.is_replica_across_hosts():
+      total_invocation_count = self._internal_ctx.num_replicas
+      replicas_consumed = 1
     else:
       total_invocation_count = self._internal_ctx.num_hosts
       replicas_consumed = self._internal_ctx.num_of_replicas_per_host
@@ -353,12 +365,14 @@ class _InternalTPUContext(object):
   def num_of_replicas_per_host(self):
     """Return the number of replicas per host."""
     if self.model_parallelism_enabled:
+      # There can be fewer replicas. This might return 0!
       return self.num_replicas // self.num_hosts
     else:
       return self.num_of_cores_per_host
 
   @property
   def num_replicas(self):
+    """Compute the total number of replicas."""
     num_cores_in_system = self.num_cores
 
     if self.model_parallelism_enabled:
@@ -405,7 +419,6 @@ class _InternalTPUContext(object):
 
   def is_input_broadcast_with_iterators(self):
     """Return true if input_fn should be run in the full_replicae config."""
-    mode = self._assert_mode()
     return ((self._config.tpu_config.per_host_input_for_training is
              tpu_config.InputPipelineConfig.BROADCAST) or
             (self.is_input_slice_broadcast_to_all_cores()))
@@ -417,6 +430,14 @@ class _InternalTPUContext(object):
             self._config.tpu_config.eval_training_input_configuration is
             tpu_config.InputPipelineConfig.SLICED)
 
+  def is_replica_across_hosts(self):
+    """Return true if single replica is across multiple hosts."""
+    # For example, when num_cores_per_replica > num_cores_per_host.
+    num_cores_per_replica = self._config.tpu_config.num_cores_per_replica
+    num_cores_per_host = self._get_tpu_system_metadata().num_of_cores_per_host
+    return (num_cores_per_replica is not None and
+            num_cores_per_replica > num_cores_per_host)
+
   def is_running_on_cpu(self, is_export_mode=False):
     """Determines whether the input_fn and model_fn should be invoked on CPU.
 
@@ -474,14 +495,13 @@ class _InternalTPUContext(object):
   def batch_size_for_input_fn(self):
     """Returns the shard batch size for `input_fn`."""
     global_batch_size = self.global_batch_size
-    if (self.is_running_on_cpu() or self.is_input_broadcast_with_iterators()
-        and not self.is_input_slice_broadcast_to_all_cores()):
+    if (self.is_running_on_cpu() or self.is_input_broadcast_with_iterators()):
       return global_batch_size
 
     # On TPU
     if self.is_input_sharded_per_core() or (
         self.is_input_per_host_with_iterators()) or (
-            self.is_input_slice_broadcast_to_all_cores()):
+            self.is_replica_across_hosts()):
       return global_batch_size // self.num_replicas
     else:
       return global_batch_size // self.num_hosts
@@ -628,16 +648,17 @@ class _InternalTPUContext(object):
 
         raise ValueError(message)
 
-    if self._config.tpu_config.num_cores_per_replica:
+    if self._config.tpu_config.num_cores_per_replica and (
+        not self.is_input_per_host_with_iterators()):
       num_cores_per_replica = self._config.tpu_config.num_cores_per_replica
       num_cores_per_host = self._get_tpu_system_metadata().num_of_cores_per_host
       if num_cores_per_replica > num_cores_per_host:
         raise ValueError(
-            'The num of cores required by the model parallelism, specified by '
-            'TPUConfig.num_cores_per_replica, is larger than the '
-            'num_cores_per_host. num_cores_per_replica: {}, '
-            'num_cores_per_host: {}'.format(num_cores_per_replica,
-                                            num_cores_per_host))
+            'Except the PER_HOST_V2 mode, the num of cores required by '
+            'model parallelism specified by TPUConfig.num_cores_per_replica '
+            'should be less than or equal to the num_cores_per_host. '
+            'num_cores_per_replica: {}, num_cores_per_host: {}'.format(
+                num_cores_per_replica, num_cores_per_host))
 
     if mode == model_fn_lib.ModeKeys.TRAIN:
       if (self._train_batch_size % num_replicas != 0 and
@@ -649,32 +670,44 @@ class _InternalTPUContext(object):
     elif mode == model_fn_lib.ModeKeys.EVAL:
       if self._eval_batch_size is None:
         raise ValueError(
-            'eval_batch_size in TPUEstimator constructor cannot be `None`'
+            'eval_batch_size in TPUEstimator constructor cannot be `None` '
             'if .evaluate is running on TPU.')
       if (self._eval_batch_size % num_replicas != 0 and
           not self.is_input_broadcast_with_iterators()):
         raise ValueError(
             'eval batch size {} must be divisible by number of replicas {}'
             .format(self._eval_batch_size, num_replicas))
-      if num_hosts > 1 and not self.is_input_broadcast_with_iterators():
+      if num_hosts != 1 and not (
+          self.is_input_broadcast_with_iterators()) and not (
+              num_replicas == 1 and self.is_input_per_host_with_iterators()):
         raise ValueError(
-            'TPUEstimator.evaluate should be running on single TPU'
-            ' instead of a Pod.')
+            'TPUEstimator.evaluate is only supported under three conditions: '
+            '1. num_hosts=1; 2. BROADCAST mode; '
+            '3. PER_HOST_V2 mode with num_replicas=1. '
+            'mode: {}; num_hosts: {}; num_replicas=1:{}'.format(
+                self._config.tpu_config.per_host_input_for_training,
+                num_hosts, num_replicas))
     else:
       assert mode == model_fn_lib.ModeKeys.PREDICT
       if self._predict_batch_size is None:
         raise ValueError(
-            'predict_batch_size in TPUEstimator constructor should not be '
-            '`None` if .predict is running on TPU.')
+            'predict_batch_size in TPUEstimator constructor cannot be `None` '
+            'if .predict is running on TPU.')
       if (self._predict_batch_size % num_replicas != 0 and
           not self.is_input_broadcast_with_iterators()):
         raise ValueError(
             'predict batch size {} must be divisible by number of replicas {}'
             .format(self._predict_batch_size, num_replicas))
-      if num_hosts > 1 and not self.is_input_broadcast_with_iterators():
+      if num_hosts != 1 and not (
+          self.is_input_broadcast_with_iterators()) and not (
+              num_replicas == 1 and self.is_input_per_host_with_iterators()):
         raise ValueError(
-            'TPUEstimator.predict should be running on single TPU worker. '
-            'got {}.'.format(num_hosts))
+            'TPUEstimator.predict is only supported under three conditions: '
+            '1. num_hosts=1; 2. BROADCAST mode; '
+            '3. PER_HOST_V2 mode with num_replicas=1. '
+            'mode: {}; num_hosts: {}; num_replicas=1:{}'.format(
+                self._config.tpu_config.per_host_input_for_training,
+                num_hosts, num_replicas))
 
     # Record the state "validated" into lazy dictionary.
     self._lazy_validation_dict[mode] = True
@@ -700,6 +733,8 @@ class _InternalTPUContext(object):
     job_device = '' if master is None else ('/job:%s' % master)
 
     num_of_replicas_per_host = self.num_of_replicas_per_host
+    assert num_of_replicas_per_host > 0, (
+        'Got num_of_replicas_per_host: {}'.format(num_of_replicas_per_host))
     host_id = replica_id / num_of_replicas_per_host
     ordinal_id = replica_id % num_of_replicas_per_host
 
@@ -736,6 +771,45 @@ class _OneCoreTPUContext(_InternalTPUContext):
     return tpu_system_metadata
 
 
+class _TPUEstimatorReplicaContext(distribute_lib.ReplicaContext):
+  """Internal context for storing replica id.
+
+  This is to set eager.context.Context() so that only summary ops from
+  0th replica is executed.
+  """
+
+  def __init__(self, replica_id_in_sync):
+    """Creates internal replica context for TPUEstimator.
+
+    Args:
+      replica_id_in_sync: Zero indexed integer id of replica that is running the
+        TPU compuation.
+    """
+    super(_TPUEstimatorReplicaContext, self).__init__(None, replica_id_in_sync)
+    # Use default strategy and replica context when variables are
+    # accessed/watched for backpropagation.
+    # pylint: disable=protected-access
+    self._thread_context = distribution_strategy_context._DefaultReplicaThreadMode(
+    )
+    self._strategy = self._thread_context.strategy
+    # pylint: enable=protected-access
+
+  def __enter__(self):
+    def replica_id_is_zero():
+      return math_ops.equal(self._replica_id_in_sync_group,
+                            constant_op.constant(0))
+
+    summary_state = summary_ops_v2._summary_state  # pylint: disable=protected-access
+    self._summary_recording_distribution_strategy = (
+        summary_state.is_recording_distribution_strategy)
+    summary_state.is_recording_distribution_strategy = replica_id_is_zero
+
+  def __exit__(self, exception_type, exception_value, traceback):
+    summary_state = summary_ops_v2._summary_state  # pylint: disable=protected-access
+    summary_state.is_recording_distribution_strategy = (
+        self._summary_recording_distribution_strategy)
+
+
 def _get_tpu_context(config, train_batch_size, eval_batch_size,
                      predict_batch_size, use_tpu, eval_on_tpu,
                      embedding_config_spec):
